@@ -386,9 +386,6 @@ void mclear_online(int y)
     mmvprintw(y, 0, space);
 }
 
-
-
-
 void mmove(int y, int x)
 {
     gX = x;
@@ -1895,18 +1892,32 @@ int mis_raw_mode()
 
 void msave_screen()
 {
+#if defined(__FREEBSD__)
+#else
     char* str = tigetstr("smcup");
     if(str) {
         ttywrite(str);
     }
+    str = tigetstr("sc");
+    if(str) {
+        ttywrite(str);
+    }
+#endif
 }
 
 void mrestore_screen()
 {
+#if defined(__FREEBSD__)
+#else
     char* str = tigetstr("rmcup");
     if(str) {
         ttywrite(str);
     }
+    str = tigetstr("rc");
+    if(str) {
+        ttywrite(str);
+    }
+#endif
 }
 
 
