@@ -429,6 +429,34 @@ static void nextin_writer(pid_t pid, sObject* nextin, int* pipeinfds, int* pipeo
     *nextin_reader_pid = pid2;
 }
 
+void make_job_title(sStatment* statment, sCommand* command_, char* title, int title_num)
+{
+    title[0] = 0;
+
+    int i;
+    for(i=0; i<statment->mCommandsNum; i++) {
+        sCommand* command = statment->mCommands + i;
+
+        int j;
+        for(j=0; j<command->mArgsNumRuntime; j++) {
+            char* arg = command->mArgsRuntime[j];
+
+            xstrncat(title, arg, BUFSIZ);
+            if(j+1 < command->mArgsNumRuntime) {
+                xstrncat(title, " ", BUFSIZ);
+            }
+        }
+
+        if(command == command_) {
+            break;
+        }
+
+        if(i+1 < statment->mCommandsNum) {
+            xstrncat(title, "|", BUFSIZ);
+        }
+    }
+}
+
 static BOOL wait_child_program(pid_t pid, pid_t nextin_reader_pid, int nextout2, sRunInfo* runinfo, char* program)
 {
     if(nextin_reader_pid != -1) {
@@ -522,14 +550,9 @@ static BOOL wait_child_program(pid_t pid, pid_t nextin_reader_pid, int nextout2,
             struct termios tty;
             tcgetattr(STDIN_FILENO, &tty);
 
-            sStatment* statment = runinfo->mStatment;
             char title[BUFSIZ];
-            title[0] = 0;
-            int i;
-            for(i=0; i<statment->mCommandsNum; i++) {
-                xstrncat(title, statment->mCommands[i].mArgsRuntime[0], BUFSIZ);
-                xstrncat(title, " ", BUFSIZ);
-            }
+            make_job_title(runinfo->mStatment, runinfo->mCommand, title, BUFSIZ);
+
             sObject* job = JOB_NEW_GC(title, pid, tty);
         }
 
@@ -561,6 +584,22 @@ static BOOL wait_child_program(pid_t pid, pid_t nextin_reader_pid, int nextout2,
                 /// exited normally ///
                 else if(WIFEXITED(status)) {
                     runinfo->mRCode = WEXITSTATUS(status);
+
+                    /// command not found ///
+                    if(runinfo->mRCode == 127) {
+                        if(gAppType == kATConsoleApp) // && nextout2 == 1)
+                        {
+                            if(tcsetpgrp(0, getpgid(0)) < 0) {
+                                perror("tcsetpgrp(xyzsh)");
+                                exit(1);
+                            }
+                        }
+
+                        char buf[BUFSIZ];
+                        snprintf(buf, BUFSIZ, "command not found");
+                        err_msg(buf, runinfo->mSName, runinfo->mSLine, program);
+                        return FALSE;
+                    }
                 }
                 /// stopped from signal ///
                 else if(WIFSTOPPED(status)) {
@@ -570,14 +609,9 @@ static BOOL wait_child_program(pid_t pid, pid_t nextin_reader_pid, int nextout2,
                     struct termios tty;
                     tcgetattr(STDIN_FILENO, &tty);
 
-                    sStatment* statment = runinfo->mStatment;
                     char title[BUFSIZ];
-                    title[0] = 0;
-                    int i;
-                    for(i=0; i<statment->mCommandsNum; i++) {
-                        xstrncat(title, statment->mCommands[i].mArgsRuntime[0], BUFSIZ);
-                        xstrncat(title, " ", BUFSIZ);
-                    }
+                    make_job_title(runinfo->mStatment, runinfo->mCommand, title, BUFSIZ);
+
                     sObject* job = JOB_NEW_GC(title, pid, tty);
                 }
 
@@ -644,15 +678,13 @@ static BOOL run_exec_cprog(sCommand* command, char* program, int nextin, int nex
     if(program_at_cd) {
         if(access(program, X_OK) == 0) {
             fprintf(stderr, "This is program at current directory\n");
-            kill(getppid(), SIGUSR1);
-            exit(1);
+            exit(127);
         }
     }
 
     execv(program, command->mArgsRuntime);
     //fprintf(stderr, "exec('%s') error\n", command->mArgsRuntime[0]);  // comment out for try
-    kill(getppid(), SIGUSR1);
-    exit(1);
+    exit(127);
 }
 
 static BOOL run_external_command(char* program, sObject* nextin, sObject* nextout, sRunInfo* runinfo)
@@ -1138,6 +1170,15 @@ static void redirect_finish(int opend_fd)
 
 static BOOL statment_tree(sStatment* statment, sObject* pipein, sObject* pipeout, sRunInfo* runinfo, sObject* current_object)
 {
+    /// error check ///
+    if(statment->mFlags & STATMENT_BACKGROUND) {
+        if(gAppType == kATOptC) {
+            err_msg("can't make a job background on script mode. you can make a job background on interactive shell", runinfo->mSName, runinfo->mSLine, NULL);
+            return FALSE;
+        }
+    }
+
+    /// head of statment ///
     sObject* nextin;
     if(statment->mFlags & STATMENT_CONTEXTPIPE) {
         /// read stdin ///
@@ -1204,6 +1245,7 @@ static BOOL statment_tree(sStatment* statment, sObject* pipein, sObject* pipeout
         nextin = pipein;
     }
 
+    /// command loop ///
     int i;
     for(i=0; i<statment->mCommandsNum; i++) {
         sCommand* command = runinfo->mCommand = statment->mCommands + i;
@@ -1395,7 +1437,6 @@ static BOOL statment_tree(sStatment* statment, sObject* pipein, sObject* pipeout
                     }
                 }
             }
-            sCommand_sweep_runtime_info(command);
         }
 
         if(opened_fd != -1) redirect_finish(opened_fd);
@@ -1408,6 +1449,8 @@ static BOOL statment_tree(sStatment* statment, sObject* pipein, sObject* pipeout
         }
     }
 
+    sStatment_sweep_runtime_info(statment);
+
     /// reverse the return code ///
     if(statment->mFlags & STATMENT_REVERSE) {
         runinfo->mRCode = !runinfo->mRCode;
